@@ -1,11 +1,11 @@
 /*
  * acct.c	Accounting routines.
  *
- *		Copyright 1994 Livingston Enterprises Inc
- *		Copyright 1999-2001 Cistron Internet Services B.V.
+ *		Copyright 1996-2001	Cistron Internet Services B.V.
+ *		Copyright 2002		Cistron IP B.V.
  */
 char acct_rcsid[] =
-"$Id: acct.c,v 1.19 2001/11/26 21:51:58 miquels Exp $";
+"$Id: acct.c,v 1.20 2002/01/22 14:13:45 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -283,7 +283,7 @@ int rad_accounting_radxtmp(AUTH_REQ *authreq, int dowtmp)
 		int check2 = 0;
 
 		/*
-		 *	ComOS (up to and including 3.5.1b20) does not send
+		 *	ComOS (up to and including 3.9.1c1) does not send
 		 *	standard PW_STATUS_ACCOUNTING_XXX messages.
 		 *
 		 *	Check for:  o no Acct-Session-Time, or time of 0
