@@ -1,11 +1,11 @@
 /*
  * attrprint.c	Functions to print A/V pairs.
  *
- *		Copyright 1992 Livingston Enterprises Inc\
- *		Copyright 1998-2001 Cistron Internet Services B.V.
+ *		Copyright 1998-2001	Cistron Internet Services B.V.
+ *		Copyright 2002		Cistron IP B.V.
  */
 char attrprint_rcsid[] =
-"$Id: attrprint.c,v 1.8 2001/11/14 16:15:00 miquels Exp $";
+"$Id: attrprint.c,v 1.10 2002/01/23 12:42:06 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -146,7 +146,7 @@ void fprint_attr_val(FILE *fd, VALUE_PAIR *pair)
 			{
 				left -= 2;
 				i = ptr[1] - 2;
-				fprintf(fd, ":T%d:L%d:", ptr[0], i);
+				fprintf(fd, ":T%d:L%d:", ptr[0], i + 2);
 				ptr += 2;
 			}
 			while (i > 0 && left > 0) {
