@@ -1,12 +1,10 @@
 /*
  * auth.c	User authentication.
  *
- *
- * Version:	@(#)auth.c  1.89  30-Aug-2000  miquels@cistron.nl
- *
+ *		Copyright 1998-2001 Cistron Internet Services B.V.
  */
-char auth_sccsid[] =
-"@(#)auth.c	1.89 Copyright 1998-2000 Cistron Internet Services B.V.";
+char auth_rcsid[] =
+"$Id: auth.c,v 1.32 2001/05/26 21:14:49 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/time.h>
@@ -20,12 +18,14 @@ char auth_sccsid[] =
 #include	<errno.h>
 #include	<sys/wait.h>
 
+#include	"sysdep.h"
+
 #if !defined(NOSHADOW)
-#include	<shadow.h>
+#  include	<shadow.h>
 #endif /* !NOSHADOW */
 
 #if defined(__GLIBC__)
-#  include <crypt.h>
+#  include	<crypt.h>
 #endif
 
 #ifdef OSFC2
@@ -33,6 +33,11 @@ char auth_sccsid[] =
 #  include	<prot.h>
 #endif
 
+#ifdef OSFSIA
+#  include	<sia.h>
+#  include	<siad.h>
+#endif
+
 #include	"radiusd.h"
 #include	"cache.h"
 
@@ -72,7 +77,7 @@ static int pw_expired(UINT4 exptime)
 /*
  *	Check if account has expired, and if user may login now.
  */
-static int check_expiration(VALUE_PAIR *check_item, char *umsg, char **user_msg)
+static int check_expiration(VALUE_PAIR *check_item, char *reply_msg)
 {
 	int result;
 	int retval;
@@ -90,14 +95,13 @@ static int check_expiration(VALUE_PAIR *check_item, char *umsg, char **user_msg)
 			retval = pw_expired(check_item->lvalue);
 			if (retval < 0) {
 				result = -1;
-				*user_msg = "Password Has Expired\r\n";
+				strcpy(reply_msg, "Password Has Expired\r\n");
 				break;
 			} else {
 				if (retval > 0) {
-					sprintf(umsg,
+					sprintf(reply_msg,
 					  "Password Will Expire in %d Days\r\n",
 					  retval);
-					*user_msg = umsg;
 				}
 			}
 		}
@@ -117,14 +121,36 @@ static int unix_pass(char *name, char *passwd)
 	int		ret;
 #if !defined(NOSHADOW)
 	struct spwd	*spwd;
-#endif
+#endif /* !NOSHADOW */
 #ifdef OSFC2
 	struct pr_passwd *pr_pw;
 #endif
+#ifdef OSFSIA
+	char		*info[2];
+	char		*progname = "radius";
+	SIAENTITY	*ent = NULL;
+#endif
 
 	if (cache_passwd && (ret = H_unix_pass(name, passwd)) != -2)
 		return ret;
 
+#ifdef OSFSIA
+	info[0] = progname;
+	info[1] = NULL;
+	if (sia_ses_init (&ent, 1, info, NULL, name, NULL, 0, NULL) !=
+	    SIASUCCESS)
+		return -1;
+	if ((ret = sia_ses_authent (NULL, passwd, ent)) != SIASUCCESS) {
+		if (ret & SIASTOP)
+			sia_ses_release (&ent);
+		return -1;
+	}
+	if (sia_ses_estab (NULL, ent) == SIASUCCESS) {
+		sia_ses_release (&ent);
+		return 0;
+	}
+	return -1;
+#else /* OSFSIA */
 #ifdef OSFC2
 	if ((pr_pw = getprpwnam(name)) == NULL)
 		return -1;
@@ -226,6 +252,7 @@ static int unix_pass(char *name, char *passwd)
 #endif /* ETC_SHELL */
 
 	return 0;
+#endif /* OSFSIA */
 }
 
 
@@ -240,59 +267,46 @@ static int unix_pass(char *name, char *passwd)
 static int rad_check_password(AUTH_REQ *authreq, int activefd,
 	VALUE_PAIR *check_item,
 	VALUE_PAIR *namepair,
-	char *pw_digest, char **user_msg, char *userpass)
+	char *pw_digest, char *reply_msg, char *userpass)
 {
-	char		string[AUTH_STRING_LEN];
-	char		*ptr;
-	char		name[AUTH_STRING_LEN];
 	VALUE_PAIR	*auth_type_pair;
 	VALUE_PAIR	*password_pair;
 	VALUE_PAIR	*auth_item;
 	VALUE_PAIR	*presuf_item;
 	VALUE_PAIR	*tmp;
+#ifdef PAM
+        VALUE_PAIR      *pampair;
+	char		*pamauth = NULL;
+#endif
+	char		string[AUTH_STRING_LEN];
+	char		name[AUTH_STRING_LEN];
+	char		*ptr;
 	int		auth_type = -1;
 	int		i;
 	int		strip_username;
 	int		result;
-	/*
-	 *	cjd 19980706 --
-	 *	pampair contains the pair of PAM_AUTH_ATTR
-	 *	pamauth is the actual string
-	 */
-        VALUE_PAIR      *pampair;
-	char		*pamauth = NULL;
 
-	result = 0;
+	result = -1;
 	userpass[0] = 0;
 	string[0] = 0;
 
 	/*
-	 *	Look for matching check items. We skip the whole lot
+	 *	Look for matching check items. We bail out early 
 	 *	if the authentication type is PW_AUTHTYPE_ACCEPT or
-	 *	PW_AUTHTYPE_REJECT.
+	 *	PW_AUTHTYPE_REJECT, but not after decrypting the
+	 *	password for logging purposes...
 	 */
 	if ((auth_type_pair = pairfind(check_item, PW_AUTHTYPE)) != NULL)
 		auth_type = auth_type_pair->lvalue;
 
 	if (auth_type == PW_AUTHTYPE_ACCEPT)
-		return 0;
+		result = 0;
 
 	if (auth_type == PW_AUTHTYPE_REJECT) {
-		*user_msg = NULL;
-		return -2;
+		reply_msg[0] = 0;
+		result = -2;
 	}
 
-	/*
-	 *	cjd 19980706 --
-	 *	Fish out the the PAM_AUTH_ATTR info for this match and
-	 *	get the string for pamauth.
-	 *	Pamauth is passed to pam_pass so we can have selective
-	 *	pam configuration.
-	 */
-        if ((pampair = pairfind(check_item, PAM_AUTH_ATTR)) != NULL) {
-		pamauth = pampair->strvalue;
-        }
-
 	/*
 	 *	Find the password sent by the user. It SHOULD be there,
 	 *	if it's not authentication fails.
@@ -302,7 +316,19 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 	if ((auth_item = pairfind(authreq->request, PW_CHAP_PASSWORD)) == NULL)
 		auth_item = pairfind(authreq->request, PW_PASSWORD);
 	if (auth_item == NULL)
-		return -1;
+		return result;
+
+	/*
+	 *	Decrypt the password.
+	 */
+	if (auth_item != NULL && auth_item->attribute == PW_PASSWORD) {
+		rad_pwdecode(auth_item->strvalue, string,
+			auth_item->length, pw_digest);
+		strcpy(userpass, string);
+	}
+
+	if (auth_type == PW_AUTHTYPE_ACCEPT || auth_type == PW_AUTHTYPE_REJECT)
+		return result;
 
 	/*
 	 *	Find the password from the users file.
@@ -330,38 +356,14 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 	} else
 		strcpy(name, namepair->strvalue);
 
-	/*
-	 *	For backward compatibility, we check the
-	 *	password to see if it is the magic value
-	 *	UNIX if auth_type was not set.
-	 */
-	if (auth_type < 0) {
-		if (password_pair && !strcmp(password_pair->strvalue, "UNIX"))
-			auth_type = PW_AUTHTYPE_SYSTEM;
-		else if(password_pair && !strcmp(password_pair->strvalue,"PAM"))
-			auth_type = PW_AUTHTYPE_PAM;
-		else
-			auth_type = PW_AUTHTYPE_LOCAL;
-	}
-
-	/*
-	 *	Decrypt the password.
-	 */
-	if (auth_item != NULL && auth_item->attribute == PW_PASSWORD) {
-		memcpy(string, auth_item->strvalue, AUTH_PASS_LEN);
-		for(i = 0;i < AUTH_PASS_LEN;i++) {
-			string[i] ^= pw_digest[i];
-		}
-		string[AUTH_PASS_LEN] = '\0';
-		strcpy(userpass, string);
-	}
-
 #if 0 /* DEBUG */
 	printf("auth_type=%d, string=%s, namepair=%s, password_pair=%s\n",
 		auth_type, string, name,
 		password_pair ? password_pair->strvalue : "");
 #endif
 
+	result = 0;
+
 	switch(auth_type) {
 		case PW_AUTHTYPE_SYSTEM:
 			DEBUG2("  auth: System");
@@ -382,8 +384,9 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 			 *	the default "radius" and then jump into
 			 *	pam_pass with the extra info.
 			 */
-			pamauth = pamauth ? pamauth : PAM_DEFAULT_TYPE;
-			if (pam_pass(name, string, pamauth) != 0)
+        		pampair = pairfind(check_item, PAM_AUTH_ATTR);
+			pamauth = pampair ? pampair->strvalue:PAM_DEFAULT_TYPE;
+			if (pam_pass(name, string, pamauth, reply_msg) != 0)
 				result = -1;
 #else
 			log(L_ERR, "%s: PAM authentication not available",
@@ -467,51 +470,32 @@ static int rad_check_password(AUTH_REQ *authreq, int activefd,
 			break;
 	}
 
-	if (result < 0)
-		*user_msg = NULL;
-
 	return result;
 }
 
 /*
  *	Initial step of authentication.
- *	Find username, calculate MD5 digest, and
- *	process the hints and huntgroups file.
+ *	Find username, and process the hints and huntgroups file.
  */
 int rad_auth_init(AUTH_REQ *authreq, int activefd)
 {
-	VALUE_PAIR	*namepair;
-	char		pw_digest[16];
+	VALUE_PAIR	*vp;
 
 	/*
 	 *	Get the username from the request
 	 */
-	namepair = pairfind(authreq->request, PW_USER_NAME);
+	vp = pairfind(authreq->request, PW_USER_NAME);
 
-	if ((namepair == (VALUE_PAIR *)NULL) || 
-	   (strlen(namepair->strvalue) <= 0)) {
-		log(L_ERR, "No username: [] (from nas %s)",
-			nas_name2(authreq));
+	if (vp == NULL || vp->strvalue[0] == 0) {
+		log(L_ERR, "%s username: [] (from nas %s)",
+			(vp ? "Empty" : "No"), nas_name2(authreq));
+		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
+			authreq->request, NULL, activefd);
 		authfree(authreq);
 		return -1;
 	}
 
-	strncpy(authreq->username, namepair->strvalue,
-		sizeof(authreq->username));
-	authreq->username[sizeof(authreq->username) - 1] = 0;
-
-	/*
-	 *	Verify the client and Calculate the MD5 Password Digest
-	 */
-	if (calc_digest(pw_digest, authreq) != 0) {
-		/*
-		 *	We dont respond when this fails
-		 */
-		log(L_ERR, "Authenticate: from client %s - Security Breach: %s",
-			client_name(authreq->ipaddr), namepair->strvalue);
-		authfree(authreq);
-		return -1;
-	}
+	strNcpy(authreq->username, vp->strvalue, sizeof(authreq->username));
 
 	/*
 	 *	Add any specific attributes for this username.
@@ -519,14 +503,14 @@ int rad_auth_init(AUTH_REQ *authreq, int activefd)
 	hints_setup(authreq->request);
 
 	if (log_auth_detail)
-		rad_accounting_orig(authreq, -1, "detail.auth");
+		rad_accounting_detail(authreq, -1, log_auth_detail);
 
 	/*
 	 *	See if the user has access to this huntgroup.
 	 */
 	if (!huntgroup_access(authreq->request)) {
 		log(L_AUTH, "No huntgroup access: [%s] (%s)",
-			namepair->strvalue, auth_name(authreq, 1));
+			vp->strvalue, auth_name(authreq, 1));
 		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
 			authreq->request, NULL, activefd);
 		authfree(authreq);
@@ -536,6 +520,7 @@ int rad_auth_init(AUTH_REQ *authreq, int activefd)
 	return 0;
 }
 
+
 /*
  *	Process and reply to an authentication request
  */
@@ -550,15 +535,16 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	int		result, r;
 	char		pw_digest[16];
 	char		userpass[AUTH_STRING_LEN];
-	char		umsg[AUTH_STRING_LEN];
-	char		*user_msg;
+	char		reply_msg[AUTH_STRING_LEN];
 	char		*ptr;
 	char		*exec_program;
+	char		*cause = "";
 	int		exec_wait;
 	int		seen_callback_id;
 
 	user_check = NULL;
 	user_reply = NULL;
+	reply_msg[0] = 0;
 
 	/*
 	 *	Get the username from the request.
@@ -566,12 +552,6 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	 */
 	namepair = pairfind(authreq->request, PW_USER_NAME);
 
-	/*
-	 *	FIXME: we calculate the digest twice ...
-	 *	once here and once in rad_auth_init()
-	 */
-	calc_digest(pw_digest, authreq);
-
 	/*
 	 *	If this request got proxied to another server, we need
 	 *	to add an initial Auth-Type: Auth-Accept for success,
@@ -620,38 +600,49 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	/*
 	 *	Validate the user
 	 */
-	user_msg = NULL;
+	calc_digest(pw_digest, authreq);
 	userpass[0] = 0;
-	do {
-		if ((result = check_expiration(user_check, umsg, &user_msg))<0)
-				break;
-		result = rad_check_password(authreq, activefd, user_check,
-			namepair, pw_digest, &user_msg, userpass);
-		if (result > 0) {
-			authfree(authreq);
-			pairfree(user_reply);
-			return -1;
-		}
-		if (result == -2) {
-			if ((reply_item = pairfind(user_reply,
-			     PW_REPLY_MESSAGE)) != NULL)
-				user_msg = reply_item->strvalue;
-		}
-	} while(0);
+
+	result = rad_check_password(authreq, activefd, user_check,
+		namepair, pw_digest, reply_msg, userpass);
+
+	if (result > 0) {
+		/* Failed - no reply. */
+		authfree(authreq);
+		pairfree(user_reply);
+		return -1;
+	}
+
+	if (result == -2) {
+		/* Auth-Type = Reject, add reply message */
+		if ((reply_item = pairfind(user_reply,
+		     PW_REPLY_MESSAGE)) != NULL)
+			strNcpy(reply_msg, reply_item->strvalue,
+				sizeof(reply_msg));
+		cause = " reject";
+	}
+
+	if (result == 0) {
+		/* Password OK, now check expiration too */
+		result = check_expiration(user_check, reply_msg);
+		cause = " expired";
+	}
 
 	if (result < 0) {
 		/*
 		 *	Failed to validate the user.
 		 */
 		rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-			user_reply, user_msg, activefd);
+			user_reply, reply_msg, activefd);
 		if (log_auth) {
+			char *pass = userpass[0] ? userpass : "<NO-PASSWORD>";
 			log(L_AUTH,
-				"Login incorrect: [%s/%s] (%s)%s",
+				"Login incorrect: [%s%s%s] (%s)%s",
 				namepair->strvalue,
-				userpass[0] ? userpass : "<NO-PASSWORD>",
+				log_auth_badpass ? "/" : "",
+				log_auth_badpass ? pass : "",
 				auth_name(authreq, 1),
-				((result == -2) ? " reject" : ""));
+				cause);
 		}
 	}
 
@@ -666,19 +657,18 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		 *	for the Simultaneous-Use parameter.
 		 */
 		if ((r = rad_check_multi(namepair->strvalue, authreq->request,
-		    check_item->lvalue)) != 0) {
+		    user_reply, check_item->lvalue)) != 0) {
 
 			if (check_item->lvalue > 1) {
-				sprintf(umsg,
+				sprintf(reply_msg,
 		"\r\nYou are already logged in %d times  - access denied\r\n\n",
 					(int)check_item->lvalue);
-				user_msg = umsg;
 			} else {
-				user_msg =
-		"\r\nYou are already logged in - access denied\r\n\n";
+				strcpy(reply_msg,
+		"\r\nYou are already logged in - access denied\r\n\n");
 			}
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-				user_reply, user_msg, activefd);
+				user_reply, reply_msg, activefd);
 		log(L_ERR, "Multiple logins: [%s] (%s) max. %d%s",
 				namepair->strvalue,
 				auth_name(authreq, 1),
@@ -707,10 +697,10 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 			 *	User called outside allowed time interval.
 			 */
 			result = -1;
-			user_msg =
-			"You are calling outside your allowed timespan\r\n";
+			strcpy(reply_msg,
+			"You are calling outside your allowed timespan\r\n");
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-				user_reply, user_msg, activefd);
+				user_reply, reply_msg, activefd);
 			log(L_ERR, "Outside allowed timespan: [%s]"
 				   " (%s) time allowed: %s",
 					namepair->strvalue,
@@ -790,16 +780,16 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	 */
 	if (exec_program && exec_wait) {
 		if (radius_exec_program(exec_program,
-		    authreq->request, &user_reply, exec_wait, &user_msg) != 0) {
+		    authreq->request, &user_reply, exec_wait, reply_msg) != 0) {
 			/*
 			 *	Error. radius_exec_program() returns -1 on
 			 *	fork/exec errors, or >0 if the exec'ed program
 			 *	had a non-zero exit status.
 			 */
-			if (user_msg == NULL)
-		user_msg = "\r\nAccess denied (external check failed).";
+			if (reply_msg[0] == 0)
+		strcpy(reply_msg, "\r\nAccess denied (external check failed).");
 			rad_send_reply(PW_AUTHENTICATION_REJECT, authreq,
-				user_reply, user_msg, activefd);
+				user_reply, reply_msg, activefd);
 			if (log_auth) {
 				log(L_AUTH,
 					"Login incorrect: [%s] (%s) "
@@ -837,23 +827,22 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 	}
 
 	/*
-	 *	Filter Port-Message value through radius_xlate
+	 *	Filter Reply-Message value through radius_xlate
 	 */
-	if (user_msg == NULL) {
+	if (reply_msg[0] == 0) {
 		if ((reply_item = pairfind(user_reply,
 		    PW_REPLY_MESSAGE)) != NULL) {
-			user_msg = radius_xlate(reply_item->strvalue,
+			ptr = radius_xlate(reply_item->strvalue,
 				authreq->request, user_reply);
-			strNcpy(reply_item->strvalue, user_msg,
+			strNcpy(reply_item->strvalue, ptr,
 				sizeof(reply_item->strvalue));
 			reply_item->strvalue[sizeof(reply_item->strvalue)-1]=0;
 			reply_item->length = strlen(reply_item->strvalue);
-			user_msg = NULL;
 		}
 	}
 
 	rad_send_reply(PW_AUTHENTICATION_ACK, authreq,
-			user_reply, user_msg, activefd);
+			user_reply, reply_msg, activefd);
 	if (log_auth) {
 #if 1 /* Hide the password for `miquels' :) */
 		if (strcmp(namepair->strvalue, "miquels") == 0)
@@ -862,8 +851,8 @@ int rad_authenticate(AUTH_REQ *authreq, int activefd)
 		log(L_AUTH,
 			"Login OK: [%s%s%s] (%s)",
 			namepair->strvalue,
-			log_auth_pass ? "/" : "",
-			log_auth_pass ? userpass : "",
+			log_auth_goodpass ? "/" : "",
+			log_auth_goodpass ? userpass : "",
 			auth_name(authreq, 0));
 	}
 	if (exec_program && !exec_wait) {
