@@ -3,9 +3,10 @@
  *
  *		Copyright 1998-2001	Cistron Internet Services B.V.
  *		Copyright 2000		Emile van Bergen (tagged attributes)
+ *		Copyright 2002		Cistron IP B.V.
  */
 char dict_rcsid[] =
-"$Id: dict.c,v 1.7 2001/11/15 23:46:04 miquels Exp $";
+"$Id: dict.c,v 1.9 2002/01/23 12:42:06 miquels Exp $";
 
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -73,10 +74,23 @@ static int addvendor(char *name, int value)
 	return 0;
 }
 
+/*
+ *	See if we know a vendor.
+ */
+int dict_vendor(UINT4 vendorpec)
+{
+	DICT_VENDOR *v;
+
+	for (v = dictionary_vendors; v; v = v->next)
+		if (v->vendorpec == vendorpec)
+			break;
+	return v ? 1 : 0;
+}
+
 /*
  *	Parse an ATTRIBUTE line.
  */
-static DICT_ATTR *parse_attr(char *buffer, int line_no,
+static DICT_ATTR *parse_attr(char *buffer, int line_no, char *dict_file,
 				int is_nmc, int *vendor_usr_seen)
 {
 	ATTR_FLAGS	flags;		/* parsed options */
@@ -98,8 +112,8 @@ static DICT_ATTR *parse_attr(char *buffer, int line_no,
 	if(sscanf(buffer, "%63s%63s%63s%63s%63s", dummystr,
 		namestr, valstr, typestr, optstr) < 4) {
 		log(L_ERR,
-	"%s: Invalid attribute on line %d of dictionary\n",
-			progname, line_no);
+	"%s: Invalid attribute on line %d of %s\n",
+			progname, line_no, dict_file);
 		return NULL;
 	}
 
@@ -124,15 +138,15 @@ static DICT_ATTR *parse_attr(char *buffer, int line_no,
 	 */
 	if (strlen(namestr) > 39) {
 		log(L_ERR|L_CONS,
-		"dict_init: Invalid name length on line %d of dictionary",
-				line_no);
+		"dict_init: Invalid name length on line %d of %s",
+				line_no, dict_file);
 			return NULL;
 	}
 
 	if (!isdigit(*valstr)) {
 		log(L_ERR|L_CONS,
-	"dict_init: Invalid value on line %d of dictionary",
-			line_no);
+	"dict_init: Invalid value on line %d of %s",
+			line_no, dict_file);
 		return NULL;
 	}
 	if (valstr[0] != '0')
@@ -154,8 +168,8 @@ static DICT_ATTR *parse_attr(char *buffer, int line_no,
 	}
 	else {
 		log(L_ERR|L_CONS,
-	"dict_init: Invalid type on line %d of dictionary",
-			line_no);
+	"dict_init: Invalid type on line %d of %s",
+			line_no, dict_file);
 		return NULL;
 	}
 
@@ -188,8 +202,8 @@ static DICT_ATTR *parse_attr(char *buffer, int line_no,
 			if (*c) {
 dict_opterr:
 				log(L_CONS|L_ERR, "dict_init: "
-				"invalid option %s on dictionary line %d",
-							s, line_no);
+				"invalid option %s on line %d of %s",
+						s, line_no, dict_file);
 				return NULL;
 			}
 			if (s[3] == '-') {
@@ -220,8 +234,8 @@ dict_opterr:
 			}
 			if (!vendor) {
 				log(L_ERR|L_CONS,
-			"dict_init: unknown vendor %s on line %d of dictionary",
-					s, line_no);
+			"dict_init: unknown vendor %s on line %d of %s",
+					s, line_no, dict_file);
 				return NULL;
 			}
 		}
@@ -282,6 +296,8 @@ int dict_init(char *dir, char *fn)
 	char		valstr[64];
 	char		attrstr[64];
 	char		buffer[256];
+	char		filenm[256];
+	char		*dict_file;
 	int		line_no;
 	int		value;
 	int		is_attrib;
@@ -294,20 +310,21 @@ int dict_init(char *dir, char *fn)
 
 	if (fn) {
 		if (fn[0] == '/')
-			strNcpy(buffer, fn, sizeof(buffer));
+			strNcpy(filenm, fn, sizeof(filenm));
 		else
-			sprintf(buffer, "%.127s/%.127s", dir, fn);
+			sprintf(filenm, "%.127s/%.127s", dir, fn);
 	} else
-		sprintf(buffer, "%.200s/%.50s", dir, RADIUS_DICTIONARY);
+		sprintf(filenm, "%.200s/%.50s", dir, RADIUS_DICTIONARY);
 
-	if((dictfd = fopen(buffer, "r")) == (FILE *)NULL) {
+	if ((dictfd = fopen(filenm, "r")) == NULL) {
 		log(L_CONS|L_ERR, "dict_init: Couldn't open dictionary: %s",
-			buffer);
-		return(-1);
+			filenm);
+		return -1;
 	}
 
 	line_no = 0;
-	while (fgets(buffer, sizeof(buffer), dictfd) != (char *)NULL) {
+	dict_file = filenm;
+	while (fgets(buffer, sizeof(buffer), dictfd) != NULL) {
 		line_no++;
 		
 		/* Skip empty space */
@@ -320,8 +337,8 @@ int dict_init(char *dir, char *fn)
 			/* Read the $INCLUDE line */
 			if(sscanf(buffer, "%63s%63s", dummystr, valstr) != 2) {
 				log(L_ERR,
-			"%s: Invalid filename on line %d of dictionary\n",
-					progname, line_no);
+			"%s: Invalid filename on line %d of %s\n",
+					progname, line_no, dict_file);
 				return(-1);
 			}
 			if (dict_init(dir, valstr) < 0)
@@ -338,7 +355,7 @@ int dict_init(char *dir, char *fn)
 			is_attrib = is_nmc = 1;
 #endif
 		if (is_attrib) {
-			attr = parse_attr(buffer, line_no,
+			attr = parse_attr(buffer, line_no, dict_file,
 				is_nmc, &vendor_usr_seen);
 			if (attr == NULL)
 				return -1;
@@ -356,8 +373,8 @@ int dict_init(char *dir, char *fn)
 			if(sscanf(buffer, "%63s%63s%63s%63s", dummystr, attrstr,
 						namestr, valstr) != 4) {
 				log(L_ERR|L_CONS,
-		"dict_init: Invalid value entry on line %d of dictionary",
-					line_no);
+		"dict_init: Invalid value entry on line %d of %s",
+					line_no, dict_file);
 				return(-1);
 			}
 
@@ -366,22 +383,22 @@ int dict_init(char *dir, char *fn)
 			 */
 			if(strlen(attrstr) > 39) {
 				log(L_ERR|L_CONS,
-		"dict_init: Invalid attribute length on line %d of dictionary",
-					line_no);
+		"dict_init: Invalid attribute length on line %d of %s",
+					line_no, dict_file);
 				return(-1);
 			}
 
 			if(strlen(namestr) > 39) {
 				log(L_ERR|L_CONS,
-		"dict_init: Invalid name length on line %d of dictionary",
-					line_no);
+		"dict_init: Invalid name length on line %d of d%s",
+					line_no, dict_file);
 				return(-1);
 			}
 
 			if(!isdigit(*valstr)) {
 				log(L_ERR|L_CONS,
-			"dict_init: Invalid value on line %d of dictionary",
-					line_no);
+			"dict_init: Invalid value on line %d of %s",
+					line_no, dict_file);
 				return(-1);
 			}
 			value = atoi(valstr);
@@ -406,8 +423,8 @@ int dict_init(char *dir, char *fn)
 			if(sscanf(buffer, "%63s%63s%63s", dummystr, attrstr,
 						valstr) != 3) {
 				log(L_ERR|L_CONS,
-		"dict_init: Invalid vendor entry on line %d of dictionary",
-					line_no);
+		"dict_init: Invalid vendor entry on line %d of %s",
+					line_no, dict_file);
 				return(-1);
 			}
 
@@ -416,15 +433,15 @@ int dict_init(char *dir, char *fn)
 			 */
 			if(strlen(attrstr) > 39) {
 				log(L_ERR|L_CONS,
-		"dict_init: Invalid attribute length on line %d of dictionary",
-					line_no);
+		"dict_init: Invalid attribute length on line %d of %s",
+					line_no, dict_file);
 				return(-1);
 			}
 
 			if(!isdigit(*valstr)) {
 				log(L_ERR|L_CONS,
-			"dict_init: Invalid value on line %d of dictionary",
-					line_no);
+			"dict_init: Invalid value on line %d of %s",
+					line_no, dict_file);
 				return(-1);
 			}
 			value = atoi(valstr);
