@@ -3,12 +3,12 @@
  * files.c	Routines to read config files into memory,
  *		and to lookup values.
  *
- *		Copyright 1992		Livingston Enterprises, Inc.
  *		Copyright 1996-2001	Cistron Internet Services B.V.
+ *		Copyright 2002		Cistron IP B.V.
  */
 
 char files_rcsid[] =
-"$Id: files.c,v 1.34 2001/11/26 21:51:59 miquels Exp $";
+"$Id: files.c,v 1.40 2002/02/06 15:23:16 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -626,7 +626,7 @@ int rad_dbm_open(void)
 		strcat(buffer, ".db");
 #endif
 #ifdef DBIF_DBM
-		if (dbmfd) close(dbmfd);
+		if (dbmfd >= 0) close(dbmfd);
 		dbmfd = -1;
 		if (dbmfile) dbmclose();
 		dbmfile = NULL;
@@ -635,12 +635,17 @@ int rad_dbm_open(void)
 		dbmfd = checkdbm(buffer, 1);
 #endif
 #ifdef DBIF_NDBM
-		if (dbmfile) dbm_close(dbmfile);
+		if (dbmfile) {
+			dbm_close(dbmfile);
+			dbmfd = -1;
+		}
 		if ((dbmfile = dbm_open(buffer, O_RDONLY, 0)) == NULL)
 			return rad_dbm_fail(buffer);
 		dbmfd = dbm_pagfno(dbmfile);
 #endif
 #ifdef DBIF_DB1
+		if (dbmfd >= 0) close(dbmfd);
+		dbmfd = -1;
 		if (dbmfile) dbmfile->close(dbmfile);
 		if ((dbmfile = dbopen(buffer, O_RDONLY, 0600,
 				      DB_HASH, NULL)) == NULL)
@@ -648,20 +653,33 @@ int rad_dbm_open(void)
 		dbmfd = checkdbm(buffer, 1);
 #endif
 #ifdef DBIF_DB2
+		if (dbmfd >= 0) close(dbmfd);
+		dbmfd = -1;
 		if (dbmfile) dbmfile->close(dbmfile, 0);
 		if (db_open(buffer, DB_HASH, DB_RDONLY, 0600, NULL,
-				NULL, &dbmfile) < 0)
+				NULL, &dbmfile) < 0) {
+			dbmfile = NULL;
 			return rad_dbm_fail(buffer);
+		}
 		dbmfd = open(buffer, O_RDONLY);
 #endif
 #ifdef DBIF_DB3
-		if (dbmfile)
+		if (dbmfd >= 0) close(dbmfd);
+		dbmfd = -1;
+		if (dbmfile) {
 			dbmfile->close(dbmfile, 0);
-		else if (db_create(&dbmfile, NULL, 0) < 0)
+			dbmfile = NULL;
+		}
+		if (db_create(&dbmfile, NULL, 0) < 0) {
+			dbmfile = NULL;
 			return rad_dbm_fail(buffer);
+		}
 		if (dbmfile->open(dbmfile, buffer, NULL, DB_HASH,
-				DB_RDONLY, 0600) < 0)
+				DB_RDONLY, 0600) < 0) {
+			dbmfile->close(dbmfile, 0);
+			dbmfile = NULL;
 			return rad_dbm_fail(buffer);
+		}
 		dbmfd = open(buffer, O_RDONLY);
 #endif
 		if (dbmfd >= 0 && fstat(dbmfd, &st) == 0)
@@ -1596,16 +1614,21 @@ int read_realms_file(char *file)
 }
 
 /*
- *	Find a realm in the REALM list.
+ *	Find a realm in the REALM list. If the realm can't be found, use
+ *	the DEFAULT realm, *unless* the realm was the NULL realm.
  */
-REALM *realm_find(char *realm)
+REALM *realm_find(char *r, int dfl)
 {
-	REALM *cl;
+	REALM	*cl;
+
+	/* Sanity check */
+	if (r == NULL || r[0] == 0) return NULL;
 
 	for(cl = realms; cl; cl = cl->next)
-		if (strcmp(cl->realm, realm) == 0)
+		if (strcmp(cl->realm, r) == 0)
 			break;
-	if (cl) return cl;
+	if (cl || !dfl || !strcmp(r, "NULL")) return cl;
+
 	for(cl = realms; cl; cl = cl->next)
 		if (strcmp(cl->realm, "DEFAULT") == 0)
 			break;
@@ -1712,8 +1735,10 @@ int read_config_files()
 
 	sprintf(buffer, "%.200s/%.50s", radius_dir, RADIUS_USERS);
 #if USE_DBX
-	if (!use_dbm && checkdbm(buffer, 0) >= 0)
+	if (!use_dbm && checkdbm(buffer, 0) >= 0) {
 		log(L_INFO|L_CONS, "DBM files found but no -b flag given - NOT using DBM");
+		use_dbm = 0;
+	}
 #endif
 	if (!use_dbm && file_read(buffer, radius_dir, &users, 1) < 0)
 		return -1;
