@@ -2,10 +2,11 @@
  * proxy.c	Proxy stuff.
  *
  *		Copyright 1999-2001	Cistron Internet Services B.V.
+ *		Copyright 2002		Cistron IP B.V.
  */
 
 char proxy_rcsid[] =
-"$Id: proxy.c,v 1.20 2001/11/15 23:46:04 miquels Exp $";
+"$Id: proxy.c,v 1.23 2002/02/06 15:23:16 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -181,6 +182,16 @@ static void passwd_recode(char *secret_key, char *vector,
 	memcpy(pass, encoded, passlen);
 }
 
+static void striprealm(VALUE_PAIR *vp)
+{
+	char	*s;
+
+	if ((s = strrchr(vp->strvalue, '@')) != NULL) {
+		*s = 0;
+		vp->length = strlen(vp->strvalue);
+	}
+}
+
 /*
  *	Relay the request to a remote server.
  *	Returns:  1 success (we reply, caller returns without replying)
@@ -190,6 +201,7 @@ static void passwd_recode(char *secret_key, char *vector,
 int proxy_send(AUTH_REQ *authreq, int activefd)
 {
 	VALUE_PAIR		*namepair;
+	VALUE_PAIR		*called_id_pair;
 	VALUE_PAIR		*vp_passwd;
 	VALUE_PAIR		*vp, *pp;
 	AUTH_HDR		*auth;
@@ -202,16 +214,21 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 	char			pw_digest[16];
 	char			saved_username[AUTH_STRING_LEN];
 	char			saved_passwd[AUTH_STRING_LEN];
+	char			called_id[AUTH_STRING_LEN+1];
 	char			*realmname;
-	char			*ptr;
+	char			*ptr, *r;
 	char			*what = "unknown";
 	unsigned short		rport;
 	int			code, total_length;
+	int			do_striprealm;
 
 	/*
 	 *	First cleanup old outstanding requests.
 	 */
 	proxy_cleanup();
+	realmname = NULL;
+	realm = NULL;
+	do_striprealm = 0;
 
 	/*
 	 *	Look up name.
@@ -223,17 +240,43 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 	strcpy(saved_username, namepair->strvalue);
 
 	/*
-	 *	Find the realm from the _end_ so that we can
-	 *	cascade realms: user@realm1@realm2.
-	 *	A NULL realm is OK.
-	 *	If not found, we treat it as usual).
+	 *	Prefix the called-station-id with a '+' and look
+	 *	it up in the realms file.
 	 */
-	ptr = authreq->username[0] ?
-		(char *)authreq->username : namepair->strvalue;
-	if ((realmname = strrchr(ptr, '@')) != NULL)
-		realmname++;
-	if ((realm = realm_find(realmname ? realmname : "NULL")) == NULL)
-		return 0;
+	called_id_pair = pairfind(authreq->request, PW_CALLED_STATION_ID);
+	if (called_id_pair != NULL && called_id_pair->strvalue[0]) {
+		called_id[0] = '+';
+		/* Same size */
+		strcpy(called_id + 1, called_id_pair->strvalue);
+		if ((realm = realm_find(called_id, 0)) != NULL)
+			realmname = called_id;
+	}
+
+	if (realm == NULL) {
+		/*
+		 *	Find the realm from the _end_ so that we can
+		 *	cascade realms: user@realm1@realm2.
+		 *	A NULL realm is OK, but an empty realm isn't.
+		 *	If not found, we treat it as usual).
+		 */
+		ptr = authreq->username[0] ?
+			(char *)authreq->username : namepair->strvalue;
+		if ((r = strrchr(ptr, '@')) != NULL)
+			r++;
+
+		/*
+		 *	Empty realms, realms starting with '+',
+		 *	and explicit NULL and DEFAULT not allowed.
+		 */
+		if (r && (r[0] == 0 || r[0] == '+' ||
+			!strcmp(r, "NULL") || !strcmp(r, "DEFAULT")))
+			return 0;
+		if (r == NULL) r = "NULL";
+		if ((realm = realm_find(r, 1)) == NULL)
+			return 0;
+		realmname = r;
+		do_striprealm = 1;
+	}
 
 	/*
 	 *	Perhaps we're getting this request via another
@@ -265,11 +308,11 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 	 *	If "hints" was not set, we have to use the original
 	 *	username as it was before hints processing.
 	 */
-	if (!realm->opts.dohints && authreq->username[0])
+	if (!realm->opts.dohints && authreq->username[0]) {
 		strNcpy(namepair->strvalue, authreq->username,
 			sizeof(namepair->strvalue));
-	if ((realmname = strrchr(namepair->strvalue, '@')) != NULL) 
-		realmname++;
+		namepair->length = strlen(namepair->strvalue);
+	}
 
 	/*
 	 *	Should we treat this as a local request ?
@@ -278,21 +321,14 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 	if (strcmp(realm->server, "LOCAL") == 0 ||
 	    (realm->opts.noauth && code == PW_AUTHENTICATION_REQUEST) ||
 	    (realm->opts.noacct && code == PW_ACCOUNTING_REQUEST)) {
-		/* Same size */
-		strcpy(namepair->strvalue, saved_username);
-		if (!realm->opts.nostrip &&
-		    ((realmname = strrchr(namepair->strvalue, '@')) != NULL)) {
-			*realmname = 0;
-			namepair->length = strlen(namepair->strvalue);
-		}
+		if (!realm->opts.nostrip)
+			striprealm(namepair);
 		return 0;
 	}
 
 	if ((client = client_find(realm->ipaddr)) == NULL) {
 		log(L_PROXY, "cannot find secret for server %s in clients file",
 			realm->server);
-		/* Same size */
-		strcpy(namepair->strvalue, saved_username);
 		return 0;
 	}
 
@@ -303,14 +339,10 @@ int proxy_send(AUTH_REQ *authreq, int activefd)
 		rport = realm->acct_port;
 
 	/* Strip realm if needed, and set authreq->realm */
-	if (realmname != NULL) {
-		if (!realm->opts.nostrip)
-			realmname[-1] = 0;
-		strNcpy(authreq->realm, realmname, sizeof(authreq->realm));
-	} else
-		authreq->realm[0] = 0;
+	if (do_striprealm && !realm->opts.nostrip)
+		striprealm(namepair);
+	strNcpy(authreq->realm, realmname, sizeof(authreq->realm));
 
-	namepair->length = strlen(namepair->strvalue);
 	secret_key = client->secret;
 	authreq->server_ipaddr = realm->ipaddr;
 
@@ -551,8 +583,7 @@ int proxy_receive(AUTH_REQ *authreq, int activefd)
 	 *	the remote server back to the NAS, for security,
 	 *	unless the remote server is designated as trusted.
 	 */
-	if ((realm = realm_find(oldreq->realm[0] ? oldreq->realm : "NULL")) &&
-	    realm->opts.trusted) {
+	if ((realm = realm_find(oldreq->realm, 1)) && realm->opts.trusted) {
 		allowed_pairs = authreq->request;
 	} else {
 		allowed_pairs = NULL;
