@@ -2,11 +2,12 @@
  * radius.c	Miscellanous generic functions.
  *
  *		Copyright 1998-2001	Cistron Internet Services B.V.
+ *		Copyright 2002		Cistron IP B.V.
  *
  */
 
 char radius_rcsid[] =
-"$Id: radius.c,v 1.17 2001/11/15 23:46:04 miquels Exp $";
+"$Id: radius.c,v 1.21 2002/01/23 12:42:06 miquels Exp $";
 
 #include	<sys/types.h>
 #include	<sys/socket.h>
@@ -431,6 +432,220 @@ int calc_acctdigest(u_char *digest, AUTH_REQ *authreq)
 	return memcmp(digest, authreq->vector, AUTH_VECTOR_LEN) ? 2 : 0;
 }
 
+/*
+ *	Decode just one attribute.
+ */
+static VALUE_PAIR *decode_attr(AUTH_REQ *authreq, u_char *ptr,
+				int attribute, int attrlen)
+{
+	VALUE_PAIR	*pair;
+	DICT_ATTR	*attr;
+	UINT4		lvalue;
+
+	if ((pair = (VALUE_PAIR *)malloc(sizeof(VALUE_PAIR))) == NULL) {
+		log(L_ERR|L_CONS, "no memory");
+		exit(1);
+	}
+	memset(pair, 0, sizeof(VALUE_PAIR));
+
+	if ((attr = dict_attrget(attribute)) == NULL) {
+		/*DEBUG("Received unknown attribute %d", attribute);*/
+		if (attribute > 0xffff)
+			sprintf(pair->name, "Attr-%d-%d", attribute >> 16,
+				attribute & 0xffff);
+		else
+			sprintf(pair->name, "Attr-%d", attribute);
+		pair->type = PW_TYPE_STRING;
+	} else {
+		strcpy(pair->name, attr->name);
+		pair->type = attr->type;
+		pair->flags = attr->flags;	/* copy flags too */
+	}
+
+	if ( attrlen >= AUTH_STRING_LEN-1 ) {
+		DEBUG("attribute %d too long, %d >= %d", attribute,
+			attrlen, AUTH_STRING_LEN);
+		free(pair);
+		return NULL;
+	}
+
+	pair->attribute = attribute;
+	pair->length = attrlen;
+	pair->next = NULL;
+	pair->operator = PW_OPERATOR_EQUAL;
+	pair->strvalue[0] = '\0';
+
+	switch (pair->type) {
+
+		case PW_TYPE_STRING:
+			/* attrlen always < AUTH_STRING_LEN */
+			memset(pair->strvalue, 0, AUTH_STRING_LEN);
+
+			/* Support tags */
+			if (pair->flags.has_tag &&
+			    (TAG_VALID(*ptr)||pair->flags.encrypt==1)) {
+				pair->flags.tag = *ptr;
+				pair->length--;
+				memcpy(pair->strvalue, ptr + 1, 
+				       pair->length);
+				/* I don't want to touch ptr or attrlen,
+				   that's why it's done like this... */
+			}
+			else {
+				memcpy(pair->strvalue, ptr, attrlen);
+				pair->flags.tag = 0;
+			}
+
+			/*
+			 * Decrypt string attributes that have a nonzero
+			 * encryption style flag. We only do this here 
+			 * for *original requests*, not for responses 
+			 * from servers we proxied to, as we don't have 
+			 * access here to the original request we sent 
+			 * to the server, of which we would need the 
+			 * request authenticator to be able to decrypt.
+			 * (Ugh! Yes sir.)
+			 */
+			if (pair->flags.encrypt &&
+			    (authreq->code == PW_AUTHENTICATION_REQUEST
+			  || authreq->code == PW_ACCOUNTING_REQUEST)) {
+				decrypt_attr(authreq->secret, 
+					     authreq->vector, pair);
+			}
+
+			debug_pair(stdout, pair);
+			break;
+
+		case PW_TYPE_INTEGER:
+		case PW_TYPE_IPADDR:
+			/* Is pair->length nowhere needed anymore? */ 
+	
+			/* Support *inserted* tags for IP addresses */
+			if (pair->flags.has_tag && 
+			    pair->type != PW_TYPE_INTEGER) {
+				pair->flags.tag = *ptr;
+				memcpy(&lvalue, ptr + 1, sizeof(UINT4));
+			}
+			else {
+				memcpy(&lvalue, ptr, sizeof(UINT4));
+			}
+			pair->lvalue = ntohl(lvalue);
+
+			/* Normal integers have the tag in the MSB */
+			if (pair->flags.has_tag &&
+			    pair->type == PW_TYPE_INTEGER) {
+				pair->flags.tag = (pair->lvalue >> 24) & 0xff;
+				pair->lvalue &= 0xffffff;
+			}
+
+			debug_pair(stdout, pair);
+			break;
+		
+		default:
+			DEBUG("	   %s (Unknown Type %d)",
+				pair->name,pair->type);
+			free(pair);
+			pair = NULL;
+			break;
+	}
+	return pair;
+}
+
+/*
+ *	Decode a list of attributes.
+ *	On the Vendor-Specific attribute, recurse (once).
+ */
+static VALUE_PAIR *decode_attrs(AUTH_REQ *authreq, u_char *ptr,
+				int length, int vendorpec)
+{
+	VALUE_PAIR	*first_pair, *prev;
+	VALUE_PAIR	*pair;
+	UINT4		lvalue;
+	UINT4		pec;
+	int		attribute;
+	int		attrlen;
+
+	first_pair = prev = NULL;
+
+	while (length > 0) {
+#ifdef ATTRIB_NMC
+		if (vendorpec == VENDORPEC_USR) {
+			if (length < 4) {
+				DEBUG("VSA(USR) attribute too short (%d),"
+					" skipped", length);
+				length = 0;
+				continue;
+			}
+			memcpy(&lvalue, ptr, 4);
+			attribute = (ntohl(lvalue) & 0xFFFF) |
+					(vendorpec << 16);
+			ptr += 4;
+			length -= 4;
+			attrlen = length;
+		} else
+#endif
+		{
+			attribute = *ptr++ | (vendorpec << 16);
+			attrlen   = *ptr++;
+			attrlen -= 2;
+			length  -= 2;
+		}
+		if (attrlen < 0) {
+			if (vendorpec) {
+				DEBUG("VSA(%d) attribute %d too short (%d),"
+				" skipped", vendorpec, attribute&0xffff,
+				attrlen+2);
+			} else {
+				DEBUG("Attribute %d too short (%d),"
+				" skipped", attribute, attrlen+2);
+			}
+			length = 0;
+			continue;
+		}
+		if ( attrlen > length ) {
+			DEBUG("attribute %d longer as buffer left, %d > %d",
+				attribute, attrlen, length);
+			length = 0;
+			continue;
+		}
+
+		/*
+		 *	This could be a Vendor-Specific attribute.
+		 *	In that case, recurse.
+		 *
+		 */
+		pec = 0;
+		if (vendorpec == 0 &&
+		    attribute == PW_VENDOR_SPECIFIC && attrlen > 6) {
+			memcpy(&lvalue, ptr, 4);
+			pec = ntohl(lvalue);
+			if (dict_vendor(pec)) {
+				ptr += 4;
+				attrlen -= 4;
+				length -= 4;
+			} else
+				pec = 0;
+		}
+
+		if (pec > 0)
+			pair = decode_attrs(authreq, ptr, attrlen, pec);
+		else
+			pair = decode_attr(authreq, ptr, attribute, attrlen);
+
+		if (pair) {
+			if (first_pair == NULL)
+				first_pair = pair;
+			else
+				prev->next = pair;
+			prev = pair;
+		}
+
+		ptr += attrlen;
+		length -= attrlen;
+	}
+	return first_pair;
+}
+
 /*
  *	Receive UDP client requests, build an authorization request
  *	structure, and attach attribute-value pairs contained in
@@ -439,26 +654,18 @@ int calc_acctdigest(u_char *digest, AUTH_REQ *authreq)
 AUTH_REQ	*radrecv(UINT4 host, u_short udp_port,
 			u_char *buffer, int length)
 {
+	AUTH_HDR	*auth;
+	AUTH_REQ	*authreq;
+	VALUE_PAIR	*pair;
 	u_char		*ptr;
 	char		hn[16];
-	AUTH_HDR	*auth;
 	int		totallen;
-	int		attribute;
-	int		attrlen;
-	int		vendorlen;
-	DICT_ATTR	*attr;
-	UINT4		lvalue;
-	UINT4		vendorpec;
-	VALUE_PAIR	*first_pair;
-	VALUE_PAIR	*prev;
-	VALUE_PAIR	*pair;
-	AUTH_REQ	*authreq;
 
 	/*
 	 *	Pre-allocate the new request data structure
 	 */
 
-	if((authreq = (AUTH_REQ *)malloc(sizeof(AUTH_REQ))) ==
+	if ((authreq = (AUTH_REQ *)malloc(sizeof(AUTH_REQ))) ==
 						(AUTH_REQ *)NULL) {
 		log(L_ERR|L_CONS, "no memory");
 		exit(1);
@@ -491,185 +698,8 @@ AUTH_REQ	*radrecv(UINT4 host, u_short udp_port,
 	 */
 	ptr = auth->data;
 	length -= AUTH_HDR_LEN;
-	first_pair = (VALUE_PAIR *)NULL;
-	prev = (VALUE_PAIR *)NULL;
 
-	vendorlen  = 0;
-
-	while(length > 0) {
-
-		if (vendorlen > 0) {
-			attribute = *ptr++ | (vendorpec << 16);
-			attrlen   = *ptr++;
-		} else {
-			attribute = *ptr++;
-			attrlen   = *ptr++;
-		}
-		if (attrlen < 2) {
-			length = 0;
-			continue;
-		}
-		attrlen -= 2;
-		length  -= 2;
-
-		/*
-		 *	This could be a Vendor-Specific attribute.
-		 *
-		 */
-		if (vendorlen <= 0 &&
-		    attribute == PW_VENDOR_SPECIFIC && attrlen > 6) {
-			memcpy(&lvalue, ptr, 4);
-			vendorpec = ntohl(lvalue);
-#ifdef ATTRIB_NMC
-			if (vendorpec == VENDORPEC_USR) {
-				ptr += 4;
-				memcpy(&lvalue, ptr, 4);
-				/*printf("received USR %04x\n", ntohl(lvalue));*/
-				attribute = (ntohl(lvalue) & 0xFFFF) |
-						(vendorpec << 16);
-				ptr += 4;
-				attrlen -= 8;
-				length -= 8;
-			} else
-#endif
-			{
-				ptr += 4;
-				vendorlen = attrlen - 4;
-				attribute = *ptr++ | (vendorpec << 16);
-				attrlen   = *ptr++;
-				attrlen -= 2;
-				length -= 6;
-			}
-		}
-
-		if ((pair = (VALUE_PAIR *)malloc(sizeof(VALUE_PAIR))) ==
-					(VALUE_PAIR *)NULL) {
-			log(L_ERR|L_CONS, "no memory");
-			exit(1);
-		}
-		memset(pair, 0, sizeof(VALUE_PAIR));
-
-		if ((attr = dict_attrget(attribute)) == (DICT_ATTR *)NULL) {
-			/*DEBUG("Received unknown attribute %d", attribute);*/
-			sprintf(pair->name, "Attr-%d", attribute);
-			pair->type = PW_TYPE_STRING;
-		} else {
-			strcpy(pair->name, attr->name);
-			pair->type = attr->type;
-			pair->flags = attr->flags;	/* copy flags too */
-		}
-
-		if ( attrlen >= AUTH_STRING_LEN-1 ) {
-			DEBUG("attribute %d too long, %d >= %d", attribute,
-				attrlen, AUTH_STRING_LEN);
-			free(pair);
-		}
-		else if ( attrlen > length ) {
-			DEBUG("attribute %d longer as buffer left, %d > %d",
-				attribute, attrlen, length);
-			free(pair);
-		}
-		else {
-			pair->attribute = attribute;
-			pair->length = attrlen;
-			pair->next = (VALUE_PAIR *)NULL;
-			pair->operator = PW_OPERATOR_EQUAL;
-			pair->strvalue[0] = '\0';
-
-			switch (pair->type) {
-
-			case PW_TYPE_STRING:
-				/* attrlen always < AUTH_STRING_LEN */
-				memset(pair->strvalue, 0, AUTH_STRING_LEN);
-
-				/* Support tags */
-                                if (pair->flags.has_tag &&
-				    (TAG_VALID(*ptr)||pair->flags.encrypt==1)) {
-                                        pair->flags.tag = *ptr;
-                                        pair->length--;
-                                        memcpy(pair->strvalue, ptr + 1, 
-					       pair->length);
-                                        /* I don't want to touch ptr or attrlen,
-					   that's why it's done like this... */
-                                }
-                                else {
-                                        memcpy(pair->strvalue, ptr, attrlen);
-                                        pair->flags.tag = 0;
-                                }
-
-                                /*
-                                 * Decrypt string attributes that have a nonzero
-				 * encryption style flag. We only do this here 
-				 * for *original requests*, not for responses 
-				 * from servers we proxied to, as we don't have 
-				 * access here to the original request we sent 
-				 * to the server, of which we would need the 
-				 * request authenticator to be able to decrypt.
-                                 * (Ugh! Yes sir.)
-                                 */
-                                if (pair->flags.encrypt &&
-                                    (authreq->code == PW_AUTHENTICATION_REQUEST
-				  || authreq->code == PW_ACCOUNTING_REQUEST)) {
-                                        decrypt_attr(authreq->secret, 
-                                                     authreq->vector, pair);
-                                }
-
-				debug_pair(stdout, pair);
-				if(first_pair == (VALUE_PAIR *)NULL) {
-					first_pair = pair;
-				}
-				else {
-					prev->next = pair;
-				}
-				prev = pair;
-				break;
-			
-			case PW_TYPE_INTEGER:
-			case PW_TYPE_IPADDR:
-				/* Is pair->length nowhere needed anymore? */ 
-
-				/* Support *inserted* tags for IP addresses */
-				if (pair->flags.has_tag && 
-				    pair->type != PW_TYPE_INTEGER) {
-					pair->flags.tag = *ptr;
-					memcpy(&lvalue, ptr + 1, sizeof(UINT4));
-				}
-				else {
-					memcpy(&lvalue, ptr, sizeof(UINT4));
-				}
-				pair->lvalue = ntohl(lvalue);
-
-				/* Normal integers have the tag in the MSB */
-				if (pair->flags.has_tag &&
-				    pair->type == PW_TYPE_INTEGER) {
-					pair->flags.tag = (pair->lvalue >> 24) &
-							  0xff;
-					pair->lvalue &= 0xffffff;
-				}
-
-				debug_pair(stdout, pair);
-				if(first_pair == (VALUE_PAIR *)NULL) {
-					first_pair = pair;
-				}
-				else {
-					prev->next = pair;
-				}
-				prev = pair;
-				break;
-			
-			default:
-				DEBUG("    %s (Unknown Type %d)",
-					pair->name,pair->type);
-				free(pair);
-				break;
-			}
-
-		}
-		ptr += attrlen;
-		length -= attrlen;
-		if (vendorlen > 0) vendorlen -= (attrlen + 2);
-	}
-	authreq->request = first_pair;
+	authreq->request = decode_attrs(authreq, ptr, length, 0);
 
 	/* hide the proxy pairs from the rest of the server */
 	pairmove2(&authreq->proxy_pairs, &authreq->request, PW_PROXY_STATE);
