@@ -2,10 +2,10 @@
  * radiusd.h	Definitions of structures, definitions, globals etc
  *		used throughout the server.
  *
- *		Copyright 1992		Livingston Enterprises, Inc.
- *		Copyright 1997,2000	Cistron Internet Services B.V.
+ *		Copyright 1996-2001	Cistron Internet Services B.V.
+ *		Copyright 2002		Cistron IP B.V.
  *
- * Version:	$Id: radiusd.h,v 1.28 2001/11/15 23:46:05 miquels Exp $
+ * Version:	$Id: radiusd.h,v 1.31 2002/01/23 12:42:07 miquels Exp $
  */
 
 #include "sysdep.h"
@@ -237,6 +237,7 @@ void		debug_pair(FILE *, VALUE_PAIR *);
 
 /* dict.c */
 int		dict_init(char *, char *);
+int		dict_vendor(UINT4 vendorpec);
 DICT_ATTR	*dict_attrget(int);
 DICT_ATTR	*dict_attrfind(char *);
 DICT_VALUE	*dict_valfind(char *, char *);
@@ -309,7 +310,7 @@ int		huntgroup_access(VALUE_PAIR *request_pairs);
 RADCLIENT	*client_find(UINT4 ipno);
 char		*client_name(UINT4 ipno);
 int		read_clients_file(char *);
-REALM		*realm_find(char *);
+REALM		*realm_find(char *, int);
 NAS		*nas_find(UINT4 ipno);
 char		*nas_name(UINT4 ipno);
 char		*nas_name2(AUTH_REQ *r);
