@@ -1,15 +1,14 @@
 /*
  * radutmp.h	Definitions for the Radius UTMP file.
  *
- * Version:	@(#)radutmp  2.00  30-Jul-1996  MvS.
+ *		Copyright 1998 Cistron Internet Services B.V.
+ *
+ * Version:	$Id: radutmp.h,v 1.2 2000/12/01 16:13:23 miquels Exp $
  */
 
 #ifndef _RADUTMP_H
 #define _RADUTMP_H
 
-#define RADUTMP "/var/log/radutmp"
-#define RADWTMP "/var/log/radwtmp"
-
 /*
  *      Types of connection.
  */
@@ -35,8 +34,10 @@
 
 struct radutmp {
   char login[32];		/* Loginname */
+				/* FIXME: extend to 48 or 64 bytes */
   int  nas_port;		/* Port on the terminal server (32 bits). */
   char session_id[8];		/* Radius session ID (first 8 bytes at least)*/
+				/* FIXME: extend to 16 or 32 bytes */
   unsigned int nas_address;	/* IP of portmaster. */
   unsigned int framed_address;	/* SLIP/PPP address or login-host. */
   int proto;			/* Protocol. */
@@ -45,7 +46,8 @@ struct radutmp {
   int type;			/* Type of entry (login/logout) */
   char porttype;		/* Porttype (I=ISDN A=Async T=Async-ISDN */
   char res1,res2,res3;		/* Fills up to one int */
-  char reserved[28];		/* 7 ints reserved */
+  char caller_id[16];		/* Calling-Station-ID */
+  char reserved[12];		/* 3 ints reserved */
 };
 
 #endif /* _RADUTMP_H */
