@@ -3,9 +3,15 @@
  *		Can also be installed as fingerd on the UNIX
  *		machine RADIUS runs on.
  *
- * Version:	@(#)radwho  1.22  18-Sep-1997  miquels@cistron.nl
+ *		Copyright 1997-2001	Cistron Internet Services B.V.
  *
+ * Author:	miquels@cistron.nl
+ * Patch:	22-Apr-1999	promera@cistron.nl
+ *		Added support for raw, comma seperated output
  */
+char *radwho_rcsid =
+"$Id: radwho.c,v 1.10 2001/05/07 20:33:45 miquels Exp $";
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -15,6 +21,7 @@
 #include <sys/stat.h>
 #include <sys/utsname.h>
 #include <time.h>
+#include <ctype.h>
 #include <netinet/in.h>
 #include "sysdep.h"
 
@@ -27,23 +34,53 @@
 #define SYS_FINGER "/usr/bin/finger"
 #define FINGER_DIR "/usr/local/lib/finger"
 
+char *progname = "radwho";
+
 /*
  *	Header above output and format.
  */
 char *hdr1 = 
-"Login    Name              What  TTY  When      From      Location";
-char *ufmt1 = "%-8.8s %-17.17s %-5.5s %-4.4s %-9.9s %-9.9s %-.16s%s";
-char *rfmt1 = "%-8.8s %-17.17s %-5.5s S%-3d %-9.9s %-9.9s %-.19s%s";
+"Login      Name              What  TTY  When      From      Location";
+char *ufmt1 = "%-10.10s %-17.17s %-5.5s %-4.4s %-9.9s %-9.9s %-.16s%s";
+char *ufmt1r = "%s,%s,%s,%s,%s,%s,%s%s";
+char *rfmt1 = "%-10.10s %-17.17s %-5.5s %s%-3d %-9.9s %-9.9s %-.19s%s";
+char *rfmt1r = "%s,%s,%s,%s%d,%s,%s,%s%s";
 
 char *hdr2 = 
-"Login     Port    What      When          From       Location";
-char *ufmt2 = "%-9.9s %-7.7d %-9.9s %-13.13s %-10.10s %-.16s%s";
-char *rfmt2 = "%-9.9s S%-5d  %-9.9s %-13.13s %-10.10s %-.28s%s";
+"Login      Port    What   When          From       Location";
+char *ufmt2 = "%-10.10s %-6.6d %-7.7s %-13.13s %-10.10s %-.16s%s";
+char *ufmt2r = "%s,%d,%s,%s,%s,%s%s";
+char *rfmt2 = "%-10.10s %s%-5d  %-6.6s %-13.13s %-10.10s %-.28s%s";
+char *rfmt2r = "%s,%s%d,%s,%s,%s,%s%s";
 
 char *eol = "\n";
 NAS *naslist;
 int showname = -1;
 int showptype = 0;
+int showcid = 0;
+
+/*
+ *	Safe popen. Ugh.
+ */
+FILE *safe_popen(char *cmd, char *mode)
+{
+	char		*p;
+	char		buf[1024];
+
+	/*
+	 *	Change all suspect characters into a space.
+	 */
+	strncpy(buf, cmd, sizeof(buf));
+	buf[sizeof(buf) - 1] = 0;
+	for (p = buf; *p; p++) {
+		if (isalnum(*p))
+			continue;
+		if (strchr("@%-_ \t+:,./", *p) == NULL)
+			*p = ' ';
+	}
+
+	return popen(buf, mode);
+}
 
 /*
  *	Read the naslist file.
@@ -68,7 +105,7 @@ static NAS *my_read_naslist_file(char *file)
 		if (buffer[0] == '#' || buffer[0] == '\n')
 			continue;
 		shortnm[0] = 0;
-		if (sscanf(buffer, "%s%s%s", hostnm, shortnm, nastype) < 2) {
+		if (sscanf(buffer, "%127s%31s%31s", hostnm, shortnm, nastype) < 2) {
 			fprintf(stderr, "%s[%d]: syntax error\n", file, lineno);
 			continue;
 		}
@@ -79,9 +116,9 @@ static NAS *my_read_naslist_file(char *file)
 		}
 
 		c->ipaddr = get_ipaddr(hostnm);
-		strcpy(c->nastype, nastype);
-		strcpy(c->shortname, shortnm);
-		strcpy(c->longname, ip_hostname(c->ipaddr));
+		strNcpy(c->nastype, nastype, sizeof(c->nastype));
+		strNcpy(c->shortname, shortnm, sizeof(c->shortname));
+		strNcpy(c->longname, ip_hostname(c->ipaddr), sizeof(c->longname));
 
 		c->next = cl;
 		cl = c;
@@ -107,7 +144,7 @@ int ffile(char *arg)
 	if (access(fn, X_OK) == 0) {
 		p = 1;
 		sprintf(fn, "exec %s/%.32s 2>&1", FINGER_DIR, arg);
-		fp = popen(fn, "r");
+		fp = safe_popen(fn, "r");
 	} else fp = fopen(fn, "r");
 
 	if (fp == NULL) return -1;
@@ -139,7 +176,7 @@ void sys_finger(char *l)
 		exit(0);
 
 	sprintf(fn, "exec %s %s", SYS_FINGER, l);
-	if ((fp = popen(fn, "r")) == NULL) {
+	if ((fp = safe_popen(fn, "r")) == NULL) {
 		printf("popen: %s\r\n", strerror(errno));
 		exit(1);
 	}
@@ -226,7 +263,8 @@ char *idletime(char *line)
 		strcpy(tty, "/dev/");
 	else
 		tty[0] = 0;
-	strcat(tty, line);
+	strncat(tty, line, 10);
+	tty[15] = 0;
 
 	tmp[0] = 0;
 	if (stat(tty, &st) == 0) {
@@ -259,13 +297,13 @@ char *ttyshort(char *tty)
 
 	if (strncmp(tty, "tty", 3) == 0) {
 		if (tty[3] >= '0' && tty[3] <= '9')
-			sprintf(tmp, "v%s", tty + 3);
+			sprintf(tmp, "v%.14s", tty + 3);
 		else
-			sprintf(tmp, "%s", tty + 3);
+			sprintf(tmp, "%.15s", tty + 3);
 		return tmp;
 	}
 	if (strncmp(tty, "vc", 2) == 0) {
-		sprintf(tmp, "v%s", tty + 2);
+		sprintf(tmp, "v%.14s", tty + 2);
 		return tmp;
 	}
 	if (strncmp(tty, "cu", 2) == 0) {
@@ -285,8 +323,11 @@ char *nasname(UINT4 ipaddr)
 	for(cl = naslist; cl; cl = cl->next)
 		if (cl->ipaddr == ipaddr)
 			break;
-	if (cl == NULL)
-		return "";
+	if (cl == NULL) {
+		static char buffer[32];
+		ipaddr2str(buffer, ipaddr);
+		return buffer;
+	}
 	if (cl->shortname[0])
 		return cl->shortname;
 	return cl->longname;
@@ -309,7 +350,7 @@ char *hostname(UINT4 ipaddr)
  */
 void usage(void)
 {
-	fprintf(stderr, "Usage: radwho [-lhfnsi]\n");
+	fprintf(stderr, "Usage: radwho [-lhfnsipcr] [-d dir] [-u file]\n");
 	fprintf(stderr, "       -l: show local (shell) users too\n");
 	fprintf(stderr, "       -h: hide shell users from radius\n");
 	fprintf(stderr, "       -f: give fingerd output\n");
@@ -317,6 +358,10 @@ void usage(void)
 	fprintf(stderr, "       -s: show full name\n");
 	fprintf(stderr, "       -i: show session ID\n");
 	fprintf(stderr, "       -p: show port type\n");
+	fprintf(stderr, "       -c: show caller ID, if available\n");
+	fprintf(stderr, "       -r: output as raw data\n");
+	fprintf(stderr, "       -d: path to radius config files\n");
+	fprintf(stderr, "       -u: path to radutmp file to use\n");
 	exit(1);
 }
 
@@ -333,14 +378,24 @@ int main(int argc, char **argv)
 	char inbuf[128];
 	char myname[128];
 	char session_id[16];
+	char *radutmp = RADUTMP;
+	char *radius_dir = RADIUS_DIR;
 	int fingerd = 0;
 	int showlocal = 0;
 	int hideshell = 0;
 	int showsid = 0;
-	char *p, *q;
-	int c;
+	int rawoutput = 0;
+	char *p, *q, *portind;
+	int c, portno;
 
-	while((c = getopt(argc, argv, "flhnsip")) != EOF) switch(c) {
+	while((c = getopt(argc, argv, "Dd:flhnsipcru:")) != EOF) switch(c) {
+		case 'D':
+			use_dns = 0;
+			break;
+
+		case 'd':
+			radius_dir = optarg;
+			break;
 		case 'f':
 			fingerd++;
 			showname = 0;
@@ -363,6 +418,16 @@ int main(int argc, char **argv)
 		case 'p':
 			showptype = 1;
 			break;
+		case 'c':
+			showcid = 1;
+			showname = 1;
+			break;
+		case 'r':
+			rawoutput = 1;
+			break;
+		case 'u':
+			radutmp = optarg;
+			break;
 		default:
 			usage();
 			break;
@@ -371,7 +436,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Read the "naslist" file.
 	 */
-	sprintf(inbuf, "%s/%s", RADIUS_DIR, RADIUS_NASLIST);
+	sprintf(inbuf, "%s/%s", radius_dir, RADIUS_NASLIST);
 	if ((naslist = my_read_naslist_file(inbuf)) == NULL)
 		exit(1);
 
@@ -406,8 +471,11 @@ int main(int argc, char **argv)
 	}
 
 	if (showlocal && (fp = fopen(UTMP_FILE, "r"))) {
-		fputs(showname ? hdr1 : hdr2, stdout);
-		fputs(eol, stdout);
+		if (rawoutput == 0)
+		{	
+			fputs(showname ? hdr1 : hdr2, stdout);
+			fputs(eol, stdout);
+		}
 		hdrdone = 1;
 
 		/*
@@ -422,7 +490,7 @@ int main(int argc, char **argv)
 			if (ut.ut_user[0] && ut.ut_line[0]) {
 #endif
 			   if (showname)
-				printf(ufmt1,
+				printf((rawoutput == 0? ufmt1: ufmt1r),
 					ut.ut_name,
 					fullname(ut.ut_name),
 					"shell",
@@ -435,7 +503,7 @@ int main(int argc, char **argv)
 					ut.ut_host,
 					myname, eol);
 			    else
-				printf(ufmt2,
+				printf((rawoutput==0? ufmt2:ufmt2r),
 					ut.ut_name,
 					ttyshort(ut.ut_line),
 					"shell",
@@ -454,7 +522,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Show the users logged in on the terminal server(s).
 	 */
-	if ((fp = fopen(RADUTMP, "r")) == NULL)
+	if ((fp = fopen(radutmp, "r")) == NULL)
 		return 0;
 
 	if (!hdrdone) {
@@ -473,19 +541,27 @@ int main(int argc, char **argv)
 
 			sprintf(session_id, "%.8s", rt.session_id);
 
+			if (!rawoutput && rt.nas_port > (showname ? 999 : 99999)) {
+				portind = ">";
+				portno = (showname ? 999 : 99999);
+			} else {
+				portind = "S";
+				portno = rt.nas_port;
+			}
 			if (showname)
-			    printf(rfmt1,
+			    printf((rawoutput == 0? rfmt1: rfmt1r),
 				rt.login,
-				showsid? session_id : fullname(rt.login),
+				showcid ? rt.caller_id :
+				(showsid? session_id : fullname(rt.login)),
 				proto(rt.proto, rt.porttype),
-				rt.nas_port,
+				portind, portno,
 				dotime(rt.time),
 				nasname(ntohl(rt.nas_address)),
 				hostname(rt.framed_address), eol);
 			else
-			    printf(rfmt2,
+			    printf((rawoutput == 0? rfmt2: rfmt2r),
 				rt.login,
-				rt.nas_port,
+				portind, portno,
 				proto(rt.proto, rt.porttype),
 				dotime(rt.time),
 				nasname(ntohl(rt.nas_address)),
